import torch
import os
from glob import glob


# This file contains functions to ensemble a folder of models and evaluate them on a test set, with included uncertainty estimation.


def load_models(folder, device):
    glob_path = os.path.join(folder, "*.pt")
    model_files = glob(glob_path)

    models = []
    model_descs = []

    for model_file in model_files:
        model = torch.load(model_file, map_location=device)
        models.append(model)
        model_descs.append(os.path.basename(model_file))

    return models, model_descs


def ensemble_predict(models, input):
    predictions = []
    for model in models:
        model.eval()
        with torch.no_grad():
            # Apply model and extract positive class predictions
            output = model(input)[:, 1]
            predictions.append(output)

    # Calculate mean and variance of predictions
    predictions = torch.stack(predictions)
    mean = predictions.mean()
    variance = predictions.var()

    return mean, variance


def ensemble_predict_strict_classes(models, input):
    predictions = []
    for model in models:
        model.eval()
        with torch.no_grad():
            # Apply model and extract prediction
            output = model(input)
            _, predicted = torch.max(output.data, 1)
            predictions.append(predicted)

    predictions = torch.stack(predictions)
    pos_votes = (predictions == 1).sum()
    neg_votes = (predictions == 0).sum()

    return pos_votes / len(models), pos_votes, neg_votes
